import argparse
from functools import cached_property

from ..__meta__ import DESCRIPTION, PROGRAMM
from ..download import Downloader
from ..utils import BColors
from ..utils.string_processing import unify
from .utils import ask_for_bool, ask_for_create


class Selection:
    def __init__(self, options: list):
        self.options = options

    def pprint(self):
        return "\n".join(f"{i}: {option}" for i, option in enumerate(self.options))

    def choose(self, input_: str):
        try:
            return self.options[int(input_)]
        except (ValueError, IndexError):
            return None


class DevelopmentCli:
    def __init__(self, args: argparse.Namespace):
        self.args = args

        if args.genre:
            self.genre = args.genre

        self.downloader: Downloader = Downloader()

    @cached_property
    def genre(self) -> str:
        """This is a cached property, which means if it isn't set in the constructor or before it is accessed,
        the program will be thrown in a shell

        Returns:
            str: the genre that should be used
        """
        option_string = f"{BColors.HEADER}Genres{BColors.ENDC}"
        genre_map = {}

        _string_list = []
        for i, genre in enumerate(self.downloader.get_existing_genres()):
            option_string += f"\n{BColors.BOLD}{i}{BColors.ENDC}: {genre}"

            genre_map[str(i)] = genre
            genre_map[unify(genre)] = genre

        genre = None
        while genre is None:
            print(option_string)
            print()

            i = input("> ") 
            u = unify(i)
            if u in genre_map:
                genre = genre_map[u]
                break

            if ask_for_create("genre", i):
                genre = i
                break

        return genre


def main():
    parser = argparse.ArgumentParser(
        prog=PROGRAMM,
        description=DESCRIPTION,
        epilog='This is just a development cli. The real frontend is coming soon.'
    )
    parser.add_argument('--genre', '-g', action='store_const')
    args = parser.parse_args()
